from dataclasses import dataclass
from enum import Enum
from time import sleep
from abc import ABC, abstractmethod
from typing import Generic, TypeVar
import traceback
from zpy.app import zapp_context as ctx
class Emoji(Enum):
    GREEN_CIRCLE = "\U0001f7e2"
    BUILDING_CONSTRUCTION = "\U0001f3d7"
    ABACOUS = "\U0001f9ee"
    HIGH_VOLTAGE = "\U000026a1"
    PARTY_POPPER = "\U0001f389"
    CROSS_MARK = "\U0000274c"
    CHECK_MARK = "\U00002705"

    def __str__(self) -> str:
        return self.value


T = TypeVar("T")


@dataclass
class OutputStep:
    data: dict
    can_continue: bool = True
    is_ok: bool = True
    message: str = ""


class Step(ABC, Generic[T]):
    def __init__(self, name: str, init_data: dict = {}):
        self.name = name
        self.init_data = init_data
        self.global_data = {}
        self.verbose = False

    @abstractmethod
    def run(self, input_data: T=None, shared_data: dict = {}) -> OutputStep:
        return OutputStep(input_data)


class Stepexecutor:
    def __init__(
        self, scenario: str, steps: list[Step], global_data: dict = {}, verbose: bool = False
    ) -> dict:
        self.scenario = scenario
        self.steps = steps
        self.global_data = global_data
        self.verbose = verbose

    def execute(self, shared_data: dict = {}) -> dict:
        print(f"\n\t{Emoji.GREEN_CIRCLE} Scenario: {self.scenario}")
        print(f"\t\t{Emoji.ABACOUS} Summary:")
        print(f"\t\t   {Emoji.HIGH_VOLTAGE} Steps: {len(self.steps)}\n")
        transaction_data = {}
        for i, step in enumerate(self.steps):
            # print(f"\t\t \U000025b6 {step.name}")
            # sleep(0.7)
            step.global_data = self.global_data
            step.verbose = self.verbose
            try:
                result = step.run(transaction_data, shared_data)
            except Exception as e:
                ctx().logger.exception(e)
                result = OutputStep({}, is_ok=False, message=str(e))

            if result and result.is_ok:
                print(f"\t\t {Emoji.CHECK_MARK} S{i+1}: {step.name}\n")
                sleep(0.2)
            if result and not result.is_ok:
                print(f"\t\t {Emoji.CROSS_MARK} S{i+1}: {step.name}\n")
                sleep(0.2)
            transaction_data = result
            self.global_data = step.global_data
        sleep(0.5)
        print(f"\n\t{Emoji.PARTY_POPPER} Finished scenario...\n")
        return transaction_data



from zpy.utils.http_client import ZHttp, ZHttpResponse, HLevels
from zpy.app import zapp_context
from zpy.utils.values import if_null_get
from zkt import Step, OutputStep

logger = zapp_context().logger


class ExtractOutputValue(Step[OutputStep]):
    def run(self, result: OutputStep = None, shared_data: dict = {}) -> dict:
        return result.data


class DictValueExtract(Step[OutputStep]):
    def __init__(self, name: str, key: str):
        Step.__init__(self, name)
        self.key = key

    def run(self, result: OutputStep = None, shared_data: dict = {}) -> OutputStep:

        if isinstance(result, dict) and self.key in result:
            return OutputStep(data=result[self.key])
        if (
            isinstance(result, OutputStep)
            and isinstance(result.data, dict)
            and self.key in result.data
        ):
            return OutputStep(data=result.data[self.key])
        return OutputStep(data={})


class StoreTransactData(Step[OutputStep]):
    def __init__(self, name: str, key: str):
        Step.__init__(self, name)
        self.key = key

    def run(self, result: OutputStep = None, shared_data: dict = {}) -> OutputStep:

        if result:
            self.global_data[self.key] = result.data
        return result


class HttpResponseVerifier(Step[ZHttpResponse]):
    def __init__(self, name: str):
        Step.__init__(self, name)

    def run(self, result: ZHttpResponse = None, shared_data: dict = {}) -> OutputStep:
        if self.verbose:
            print("Response verifier: ", result.json)

        if result and result.is_ok():
            return OutputStep(data=result.json())

        return OutputStep(data=result.json(), is_ok=False)


class PostHttpRequest(Step):
    def __init__(self, name: str, log_levels: list[HLevels] = None):
        Step.__init__(self, name)
        self.log_levels = if_null_get(log_levels, [])

    def run(
        self, transaction_data: OutputStep = None, shared_data: dict = {}
    ) -> ZHttpResponse:
        x_data = {}
        if transaction_data:
            x_data = transaction_data.data

        url = x_data.get("url")
        headers = x_data.get("headers", {})
        data = x_data.get("data", {})
        body = x_data.get("body", None)
        params = x_data.get("params", {})

        if body:
            result = ZHttp.post(
                url=url,
                headers=headers,
                params=params,
                json=body,
                wrap_in_zhttp=True,
                log_options=self.log_levels,
                logger=logger,
            )
        if data:
            result = ZHttp.post(
                url=url,
                headers=headers,
                params=params,
                data=data,
                wrap_in_zhttp=True,
                log_options=self.log_levels,
                logger=logger,
            )
        return result



from setuptools import setup, find_packages


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="zkt",
    version="1.0.0",
    license="MIT",
    author="Noé Cruz",
    author_email="contactozurckz@gmail.com",
    description="Helper tool for api test automation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages("src"),
    package_dir={"": "src"},
    url="https://github.com/gmyrianthous/example-publish-pypi",
    keywords="zurckz test testing",
    install_requires=[
        "zpy-api-core",
    ],
    python_requires=">=3.6",
)
